@@ -1,5 +1,5 @@
 #
-# $Id: VRMLFields.pm,v 1.12 2001/07/11 20:43:05 ayla Exp $
+# $Id: VRMLFields.pm,v 1.36 2003/12/04 18:33:57 crc_canada Exp $
 #
 # Copyright (C) 1998 Tuomas J. Lukka 1999 John Stewart CRC Canada
 # DISTRIBUTED WITH NO WARRANTY, EXPRESS OR IMPLIED.
@@ -11,6 +11,95 @@
 # SFNode is in Parse.pm
 #
 # $Log: VRMLFields.pm,v $
+# Revision 1.36  2003/12/04 18:33:57  crc_canada
+# Basic threading ok
+#
+# Revision 1.35  2003/09/16 14:57:24  crc_canada
+# EAI in C updates for Sept 15 2003
+#
+# Revision 1.34  2003/06/12 19:08:56  crc_canada
+# more work on getting javascript routing into C
+#
+# Revision 1.33  2003/06/02 18:21:40  crc_canada
+# more work on CRoutes for Scripting
+#
+# Revision 1.32  2003/05/28 14:18:35  crc_canada
+# Scripts moved to CRoute structure
+#
+# Revision 1.31  2003/05/08 16:01:33  crc_canada
+# Moving code to C
+#
+# Revision 1.30  2003/04/25 19:43:13  crc_canada
+# changed SFTime to double from float - float was not enough precision
+# to hold time since epoch values.
+#
+# Revision 1.29  2003/03/20 18:37:22  ayla
+#
+# Added init() functions to be used by VRML::NodeIntern::do_defaults() to
+# supply eventOuts with default values (used by Script, PROTO definitions).
+#
+# Revision 1.28  2003/01/28 18:23:59  ayla
+# Problem with checking for undefined values in VRML::Field::Multi::as_string fixed.
+#
+# Revision 1.27  2002/11/29 17:07:10  ayla
+#
+# Parser was choking on escaped double quotes inside strings.  This should
+# make it better.
+#
+# Revision 1.26  2002/11/28 20:15:41  crc_canada
+# For 0.37, PixelTextures are handled in the same fashion as other static images
+#
+# Revision 1.25  2002/11/25 16:55:27  ayla
+#
+# Tweaked float/double formatting to strings and made changes to property
+# setting for SFNodes.
+#
+# Revision 1.24  2002/11/22 22:09:14  ayla
+# Tweaking SFFloat and SFTime formatting in as_string.
+#
+# Revision 1.23  2002/11/22 16:28:57  ayla
+#
+# Format floating point numbers for string conversion.
+#
+# Revision 1.22  2002/11/20 21:33:55  ayla
+#
+# Modified as_string functions as needed, commented unused js functions.
+#
+# Revision 1.21  2002/09/19 19:39:35  crc_canada
+# some changes brought about by much EAI work.
+#
+# Revision 1.20  2002/07/11 16:21:42  crc_canada
+# ImageTexture fields changed to read in textures by C rather than Perl
+#
+# Revision 1.19  2002/06/21 19:51:54  crc_canada
+# Comma handling in number strings is much better now; previous code would
+# not handle commas when preceeded by a space.
+#
+# Revision 1.18  2002/06/17 16:39:19  ayla
+#
+# Fixed VRML::DEF copy problem in package VRML::Field.
+#
+# Revision 1.17  2002/05/22 21:47:52  ayla
+#
+# Broke Scene.pm into files containing constituent packages for sanity's sake.
+#
+# Revision 1.16  2002/05/01 15:12:24  crc_canada
+# add __texture field to store OpenGLs bound texture number
+#
+# Revision 1.15  2002/01/12 15:44:18  hoenicke
+# Removed the Java SAI stuff from here.  It moved to VRMLJava.pm
+# and java/classes/vrml/genfields.pl
+#
+# Revision 1.14  2001/11/28 08:34:30  ayla
+#
+# Tweaked regexp processing for VRML::Field::SFString::parse due to a submitted
+# bug report.  It appeared that perl crashed with a segmentation fault while
+# attempting to process the regexp for an SFString.  This process has
+# been optimized.  Hopefully it won't break anything!
+#
+# Revision 1.13  2001/08/16 16:55:54  crc_canada
+# Viewpoint work
+#
 # Revision 1.12  2001/07/11 20:43:05  ayla
 #
 #
@@ -104,6 +193,7 @@ sub es {
 # By encapsulating things well enough, we'll be able to completely
 # change the interface later, e.g. to fit together with javascript etc.
 sub ctype ($) {die "VRML::Field::ctype - abstract function called"}
+sub clength ($) {0} #for C routes. Keep in sync with getClen in VRMLC.pm. 
 sub calloc ($$) {""}
 sub cassign ($$) {"$_[1] = $_[2];"}
 sub cfree ($) {if($_[0]->calloc) {return "free($_[1]);"} return ""}
@@ -111,23 +201,25 @@ sub cget {if(!defined $_[2]) {return "$_[1]"}
 	else {die "If CGet with indices, abstract must be overridden"} }
 sub cstruct () {""}
 sub cfunc {die("Must overload cfunc")}
-sub jsimpleget {return {}}
 
 sub copy {
 	my($type, $value) = @_;
-	if(!ref $value) {return $value}
-	if(ref $value eq "ARRAY") {
+
+	if (!ref $value) {
+		return $value
+	} elsif (ref $value eq "ARRAY") {
 		return [map {copy("",$_)} @$value]
-	}
-	if(ref $value eq "VRML::Node") {
+	} elsif (ref $value eq "VRML::NodeIntern") {
 		return $value;
-	}
-	if(ref $value eq "VRML::DEF") {
-		my $ret = [map {copy("",$_)} @$value];
-		bless $ret, "VRML::DEF";
+	} elsif (ref $value eq "VRML::DEF") {
+		##my $ret = [map {copy("",$_)} @$value];
+		##bless $ret, "VRML::DEF";
+		##return $ret;
+		my $ret = bless { map {copy("",$_)} %$value }, "VRML::DEF";
 		return $ret;
+	} else {
+		die("Can't copy this");
 	}
-	die("Can't copy this");
 }
 
 sub as_string {"VRML::Field - can't print this!"}
@@ -138,6 +230,8 @@ package VRML::Field::SFFloat;
 @ISA=VRML::Field;
 VRML::Error->import();
 
+sub init { return 0.0; }
+
 sub parse {
 	my($type,$p,$s,$n) = @_;
 	$_[2] =~ /\G\s*($Float)/ogcs or 
@@ -145,58 +239,30 @@ sub parse {
 	return $1;
 }
 
-sub as_string {$_[1]}
+sub as_string { return sprintf("%.4g", $_[1]); }
 
 sub print {print $_[1]}
 
 sub ctype {"float $_[1]"}
+sub clength {2} #for C routes. Keep in sync with getClen in VRMLC.pm. 
 sub cfunc {"$_[1] = SvNV($_[2]);\n"}
 
 
-# JAVA SAI stuff...
-
-sub jdata {"float v;"}
-sub jalloc {""}
-sub jset {return {""=>"v = 0;", "float val" => "v=val;"}}
-sub jset_str { '
-	s = s.trim();
-	v = new Float(s).floatValue();
-'}
-sub jget {return {float => "return v;"}}
-sub jcopy {"v = f.getValue();"}
-sub jstr {"return new Float(v).toString();"}
-sub jclonearg {"v"}
-sub toj {$_[1]}
-sub fromj {$_[1]}
-
-
-# end of JAVA SAI stuff...
-
-
 ###########################################################
 package VRML::Field::SFTime;
 @ISA=VRML::Field::SFFloat;
 
-
-# JAVA SAI stuff...
-
-sub jdata {"double v;"}
-sub jset {return {""=>"v = 0;", "double val" => "v=val;"}}
-sub jset_str { '
-	s = s.trim();
-	v = new Double(s).doubleValue();
-'}
-sub jget {return {double => "return v;"}}
-sub jstr {"return new Double(v).toString();"}
-
-# end of JAVA SAI stuff...
-
+sub as_string { return sprintf("%f", $_[1]); }
+sub ctype {"double $_[1]"}
+sub clength {3} #for C routes. Keep in sync with getClen in VRMLC.pm.
 
 ###########################################################
 package VRML::Field::SFInt32;
 @ISA=VRML::Field;
 VRML::Error->import;
 
+sub init { return 0; }
+
 sub parse {
 	my($type,$p,$s,$n) = @_;
 	$_[2] =~ /\G\s*($Integer)\b/ogsc 
@@ -208,38 +274,21 @@ sub print {print " $_[1] "}
 sub as_string {$_[1]}
 
 sub ctype {return "int $_[1]"}
+sub clength {1} #for C routes. Keep in sync with getClen in VRMLC.pm. 
 sub cfunc {return "$_[1] = SvIV($_[2]);\n"}
 
-# JAVA SAI stuff...
-
-sub jdata {"int v;"}
-sub jalloc {""}
-sub jset {return {""=>"v = 0;", "int val" => "v=val;"}}
-# JAS sub jeaiset {return "int val"}
-sub jset_str { '
-	s = s.trim();
-	v = new Integer(s).intValue();
-'}
-sub jget {return {int => "return v;"}}
-sub jcopy {"v = f.getValue();"}
-sub jstr {"return new Integer(v).toString();"}
-sub jclonearg {"v"}
-sub toj {$_[1]}
-sub fromj {$_[1]}
-
-# end of JAVA SAI stuff...
-
 
 ###########################################################
 package VRML::Field::SFColor;
 @ISA=VRML::Field;
 VRML::Error->import;
 
+sub init { return [0, 0, 0]; }
+
 sub parse {
 	my($type,$p) = @_;
 
-	$_[2] =~ /\G\s*($Float)\s+($Float)\s+($Float)/ogsc
-	    or $_[2] =~ /\G\s*($Float),\s+($Float),\s+($Float)/ogsc 
+	$_[2] =~ /\G\s{0,}($Float)\s{0,},{0,1}\s{0,}($Float)\s{0,},{0,1}\s{0,}($Float)/ogsc
 	    or parsefail($_[2],"Didn't match SFColor");
 
 	return [$1,$2,$3];
@@ -251,6 +300,7 @@ sub as_string {join ' ',@{$_[1]}}
 sub cstruct {return "struct SFColor {
 	float c[3]; };"}
 sub ctype {return "struct SFColor $_[1]"}
+sub clength {5} #for C routes. Keep in sync with getClen in VRMLC.pm. 
 sub cget {return "($_[1].c[$_[2]])"}
 
 sub cfunc {
@@ -282,82 +332,12 @@ sub cfunc {
 	"
 }
 
-# JAVA SAI stuff...
-
-sub jdata {"float red,green,blue;"}
-sub jalloc {""}
-sub jset {return {"" => "red=0; green=0; blue=0;",
-	"float colors[]" => "red = colors[0]; green=colors[1]; blue=colors[2];",
-	"float r,float g,float b" => "red=r; green=g; blue=b;"
-}}
-# JAS sub jeaiset { return "float colors[]" }
-sub jset_str {"
-   	StringTokenizer tok = new StringTokenizer(s);
-	red = 	new Float(tok.nextToken()).floatValue();
-	green =	new Float(tok.nextToken()).floatValue();
-	blue =	new Float(tok.nextToken()).floatValue();
-	"
-}
-sub jget {return {"void" => ["float colors[]",
-	"colors[0] = red; colors[1] = green; colors[2] = blue;"]}
-}
-sub jcopy {"red = f.getRed(); green = f.getGreen(); blue = f.getBlue();"}
-sub jstr {'return Float.toString(red) + " " + 
-	Float.toString(green) + " " + Float.toString(blue);'}
-sub jclonearg {"red,green,blue"}
-
-sub jsimpleget {return {red => float, green => float, blue => float}}
-sub toj {join ' ',@{$_[1]}}
-sub fromj {[split ' ',$_[1]]}
-
-# end of JAVA SAI stuff...
-
-# javascript
-
-sub jsprop {
-	return '{"r", 0, JSPROP_ENUMERATE},{"g", 1, JSPROP_ENUMERATE},
-		{"b", 2, JSPROP_ENUMERATE}'
-}
-sub jsnumprop {
-	return { map {($_ => "$_[1].c[$_]")} 0..2 }
-}
-sub jstostr {
-	return "
-		{static char buff[250];
-		 sprintf(buff,\"\%f \%f \%f\", $_[1].c[0], $_[1].c[1], $_[1].c[2]);
-		 \$RET(buff);
-		}
-	"
-}
-sub jscons {
-	return [
-		"jsdouble pars[3];",
-		"d d d",
-		"&(pars[0]),&(pars[1]),&(pars[2])",
-		"$_[1].c[0] = pars[0]; $_[1].c[1] = pars[1]; $_[1].c[2] = pars[2];",
-		# Last: argless
-		"$_[1].c[0] = 0; $_[1].c[1] = 0; $_[1].c[2] = 0;",
-	];
-}
-
-sub js_default {
-	return "new SFColor(0,0,0)"
-}
-
 
 ###########################################################
 package VRML::Field::SFVec3f;
 @ISA=VRML::Field::SFColor;
 sub cstruct {return ""}
 
-sub jsprop {
-	return '{"x", 0, JSPROP_ENUMERATE},{"y", 1, JSPROP_ENUMERATE},
-		{"z", 2, JSPROP_ENUMERATE}'
-}
-sub js_default {
-	return "new SFVec3f(0,0,0)"
-}
-
 sub vec_add { join '',map {"$_[3].c[$_] = $_[1].c[$_] + $_[2].c[$_];"} 0..2; }
 sub vec_subtract { join '',map {"$_[3].c[$_] = $_[1].c[$_] - $_[2].c[$_];"} 0..2; }
 sub vec_negate { join '',map {"$_[2].c[$_] = -$_[1].c[$_];"} 0..2; }
@@ -387,11 +367,12 @@ package VRML::Field::SFVec2f;
 @ISA=VRML::Field;
 VRML::Error->import();
 
+sub init { return [0, 0]; }
+
 sub parse {
 	my($type,$p) = @_;
 
-	$_[2] =~ /\G\s*($Float)\s+($Float)/ogsc
-	    or $_[2] =~ /\G\s*($Float),\s+($Float)/ogsc
+	$_[2] =~ /\G\s{0,}($Float)\s{0,},{0,1}\s{0,}($Float)/ogsc
 	    or parsefail($_[2],"didn't match SFVec2f");
 	return [$1,$2];
 }
@@ -402,6 +383,7 @@ sub as_string {join ' ',@{$_[1]}}
 sub cstruct {return "struct SFVec2f {
 	float c[2]; };"}
 sub ctype {return "struct SFVec2f $_[1]"}
+sub clength {6} #for C routes. Keep in sync with getClen in VRMLC.pm. 
 sub cget {return "($_[1].c[$_[2]])"}
 
 sub cfunc {
@@ -431,54 +413,19 @@ sub cfunc {
 }
 
 
-# JAVA SAI stuff...
-
-sub jdata {"float x,y;"}
-sub jalloc {""}
-sub jset {return {"" => "x=0; y=0;",
-	"float coords[]" => "x = coords[0]; y=coords[1];",
-	"float x2,float y2" => "x=x2; y=y2;"
-}}
-# JAS sub jeaiset { "float coords[]" }
-sub jset_str {"
-   	StringTokenizer tok = new StringTokenizer(s);
-	x = 	new Float(tok.nextToken()).floatValue();
-	y =	new Float(tok.nextToken()).floatValue();
-	"
-}
-sub jget {return {"void" => ["float coords[]",
-	"coords[0] = x; coords[1] = y;"]}
-}
-sub jcopy {"x = f.getX(); y = f.getY();"}
-sub jstr {'return Float.toString(x) + " " + 
-	Float.toString(y) ;'}
-sub jclonearg {"x,y"}
-
-sub jsimpleget {return {x => float, y => float}}
-sub toj {join ' ',@{$_[1]}}
-sub fromj {[split ' ',$_[1]]}
-
-
-# end of JAVA SAI stuff...
-
-
-
 ###########################################################
 package VRML::Field::SFRotation;
 @ISA=VRML::Field;
 VRML::Error->import();
 
+sub init { return [0, 0, 1, 0]; }
+
 sub parse {
 	my($type,$p) = @_;
 
-	# first, look for 4 floats, NO COMMAS.
-	my $count=($_[2] =~ /\G\s*($Float)\s+($Float)\s+($Float)\s+($Float)/ogsc );
-
-	# XXX - Verify second, did not parse, maybe the user put commas in?
-	if (!$count) {
-	    $_[2] =~ /\G\s*($Float),\s+($Float),\s+($Float),\s+($Float)/ogsc 
+	# first, look for 4 floats, can have commas
+	$_[2] =~ /\G\s{0,}($Float)\s{0,},{0,1}\s{0,}($Float)\s{0,},{0,1}\s{0,}($Float)\s{0,},{0,1}\s{0,}($Float)/ogsc
 		or VRML::Error::parsefail($_[2],"not proper rotation");
-	}
 	return [$1,$2,$3,$4];
 }
 
@@ -510,7 +457,6 @@ sub rot_multvec {
 	qq~
 		double rl = AVECLEN($_[1].r);
 		double vl = AVECLEN($_[2].c);
-		/* Unused JAS double rlpt = AVECPT($_[1].r, $_[2].c) / rl / vl; */
 		float c1[3];
 		float c2[3];
 		double s = sin($_[1].r[3]), c = cos($_[1].r[3]);
@@ -529,6 +475,7 @@ sub rot_multvec {
 }
 
 sub ctype {return "struct SFRotation $_[1]"}
+sub clength {4} #for C routes. Keep in sync with getClen in VRMLC.pm. 
 sub cget {return "($_[1].r[$_[2]])"}
 
 sub cfunc {
@@ -561,199 +508,57 @@ sub cfunc {
 	"
 }
 
-sub jsprop {
-	return '{"x", 0, JSPROP_ENUMERATE},{"y", 1, JSPROP_ENUMERATE},
-		{"z", 2, JSPROP_ENUMERATE},{"angle",3, JSPROP_ENUMERATE}'
-}
-sub jsnumprop {
-	return { map {($_ => "$_[1].r[$_]")} 0..3 }
-}
-sub jstostr {
-	return "
-		{static char buff[250];
-		 sprintf(buff,\"\%f \%f \%f \%f\", $_[1].r[0], $_[1].r[1], $_[1].r[2], $_[1].r[3]);
-		 \$RET(buff);
-		}
-	"
-}
-sub jscons {
-return ["",qq~
-	jsdouble pars[4];
-	JSObject *ob1;
-	JSObject *ob2;
-	if(JS_ConvertArguments(cx,argc,argv,"d d d d",
-		&(pars[0]),&(pars[1]),&(pars[2]),&(pars[3])) == JS_TRUE) {
-		$_[1].r[0] = pars[0]; 
-		$_[1].r[1] = pars[1]; 
-		$_[1].r[2] = pars[2]; 
-		$_[1].r[3] = pars[3];
-	} else if(JS_ConvertArguments(cx,argc,argv,"o o",
-		&ob1,&ob2) == JS_TRUE) {
-		TJL_SFVec3f *vec1;
-		TJL_SFVec3f *vec2;
-		double v1len, v2len, v12dp;
-		    if (!JS_InstanceOf(cx, ob1, &cls_SFVec3f, argv)) {
-			die("sfrot obj: has to be SFVec3f ");
-			return JS_FALSE;
-		    }
-		    if (!JS_InstanceOf(cx, ob2, &cls_SFVec3f, argv)) {
-			die("sfrot obj: has to be SFVec3f ");
-			return JS_FALSE;
-		    }
-		vec1 = JS_GetPrivate(cx,ob1);
-		vec2 = JS_GetPrivate(cx,ob2);
-		v1len = sqrt( vec1->v.c[0] * vec1->v.c[0] + 
-			vec1->v.c[1] * vec1->v.c[1] + 
-			vec1->v.c[2] * vec1->v.c[2] );
-		v2len = sqrt( vec2->v.c[0] * vec2->v.c[0] + 
-			vec2->v.c[1] * vec2->v.c[1] + 
-			vec2->v.c[2] * vec2->v.c[2] );
-		v12dp = vec1->v.c[0] * vec2->v.c[0] + 
-			vec1->v.c[1] * vec2->v.c[1] + 
-			vec1->v.c[2] * vec2->v.c[2] ;
-		$_[1].r[0] = 
-			vec1->v.c[1] * vec2->v.c[2] - 
-			vec2->v.c[1] * vec1->v.c[2];
-		$_[1].r[1] = 
-			vec1->v.c[2] * vec2->v.c[0] - 
-			vec2->v.c[2] * vec1->v.c[0];
-		$_[1].r[2] = 
-			vec1->v.c[0] * vec2->v.c[1] - 
-			vec2->v.c[0] * vec1->v.c[1];
-		v12dp /= v1len * v2len;
-		$_[1].r[3] = 
-			atan2(sqrt(1-v12dp*v12dp),v12dp);
-		/* 
-		printf("V12cons: (%f %f %f) (%f %f %f) %f %f %f (%f %f %f : %f)\n",
-			vec1->v.c[0], vec1->v.c[1], vec1->v.c[2],
-			vec2->v.c[0], vec2->v.c[1], vec2->v.c[2],
-			v1len, v2len, v12dp, 
-			$_[1].r[0], $_[1].r[1], $_[1].r[2], $_[1].r[3]);
-		*/
-	} else if(JS_ConvertArguments(cx,argc,argv,"o d",
-		&ob1,&(pars[0])) == JS_TRUE) {
-		TJL_SFVec3f *vec;
-		    if (!JS_InstanceOf(cx, ob1, &cls_SFVec3f, argv)) {
-			die("multVec: has to be SFVec3f ");
-			return JS_FALSE;
-		    }
-		vec = JS_GetPrivate(cx,ob1);
-		$_[1].r[0] = vec->v.c[0]; 
-		$_[1].r[1] = vec->v.c[1]; 
-		$_[1].r[2] = vec->v.c[2]; 
-		$_[1].r[3] = pars[0];
-		
-	} else if(argc == 0) {
-		$_[1].r[0] = 0;
-		$_[1].r[0] = 0;
-		$_[1].r[0] = 1;
-		$_[1].r[0] = 0;
-	} else {
-		die("Invalid constructor for SFRotation");
-	}
-
-~];
-}
-
-sub js_default {
-	return "new SFRotation(0,0,1,0)"
-}
-# JAVA SAI stuff...
-
-sub jdata {"float myx,myy,myz,angle;"}
-sub jalloc {""}
-sub jset {return {"" => "myx=0; myy=0; myz=1; angle=0;",
-	"float rot[]" => "myx = rot[0]; myy=rot[1]; myz=rot[2];angle=rot[3];",
-	"float r,float g,float b,float a" => "myx=r; myy=g; myz=b;angle=a;"
-}}
-sub jset_str {"
-   	StringTokenizer tok = new StringTokenizer(s);
-	myx = 	new Float(tok.nextToken()).floatValue();
-	myy =	new Float(tok.nextToken()).floatValue();
-	myz =	new Float(tok.nextToken()).floatValue();
-	angle =	new Float(tok.nextToken()).floatValue();
-	"
-}
-sub jget {return {"void" => ["float rot[]",
-	"rot[0] = myx; rot[1] = myy; rot[2] = myz; rot[3] = angle;"]}
-}
-sub jcopy {"myx = f.getRed(); myy = f.getGreen(); myz = f.getBlue();",
-		"angle=getAngle();"}
-sub jstr {'return Float.toString(myx) + " " + 
-	Float.toString(myy) + " " + Float.toString(myz) + " " +
-	Float.toString(angle);'}
-sub jclonearg {"myx,myy,myz,angle"}
-
-sub jsimpleget {return {myx => float, myy => float, 
-		myz => float, angle => float}}
-sub toj {join ' ',@{$_[1]}}
-sub fromj {[split ' ',$_[1]]}
-
-# end of JAVA SAI stuff...
 
 ###########################################################
 package VRML::Field::SFBool;
 @ISA=VRML::Field;
 
+sub init { return 0; }
+
 sub parse {
 	my($type,$p,$s,$n) = @_;
-	$_[2] =~ /\G\s*(TRUE|FALSE)\b/gs or die "Invalid value for BOOL\n";
-	return ($1 eq "TRUE");
+	$_[2] =~ /\G\s*(TRUE|FALSE)\b/gs
+		or die "Invalid value for SFBool\n";
+	if ($1 eq "TRUE") {return 1};
+	return 0;
 }
 
 sub ctype {return "int $_[1]"}
+sub clength {1} #for C routes. Keep in sync with getClen in VRMLC.pm. 
 sub cget {return "($_[1])"}
 sub cfunc {return "$_[1] = SvIV($_[2]);\n"}
 
 sub print {print ($_[1] ? TRUE : FALSE)}
-sub as_string {($_[1] ? TRUE : FALSE)}
-
-# The java interface
-
-# JAVA SAI stuff...
-
-sub jdata {"boolean v;"}
-sub jalloc {""}
-sub jset {return {"" => "v = false;",
-	"boolean value" => "v = value;",
-}}
-sub jset_str { q~
-   	s = s.trim();
-	if(s.equals("1")) {v = true;}
-	else if(s.equals("0") || s.equals("")) {v = false;}
-	else {throw new Exception("Invalid boolean '"+s+"'");}
-~}
-sub jget {return {"boolean" => "return v;"}}
-sub jcopy {"v = f.getValue();"}
-sub jstr {'return (v? "1": "0");'}
-sub jclonearg {"v"}
-sub toj {return $_[1]}
-sub fromj {return $_[1]}
-
-sub js_default {return "false"}
-
-
-# end of JAVA SAI stuff...
+sub as_string {
+	my ($this, $bool, $as_ecmascript) = @_;
+	if ($as_ecmascript) {
+		return ($bool ? true : false);
+	} else {
+		return ($bool ? TRUE : FALSE);
+	}
+}
 
 
 ###########################################################
 package VRML::Field::SFString;
 @ISA=VRML::Field;
 
+# Ayla... This is closer to the VRML97 specification,
+# which allows anything but unescaped " or \ in SFStrings.
+
+$Chars = qr/(?:[\x00-\x21\x23-\x5b\x5d-\x7f]*|\x5c\x22|\x5c{2}[^\x22])*/o;
+
+sub init { return ""; }
+
 # XXX Handle backslashes in string properly
 sub parse {
-	my($type,$p,$s,$n) = @_;
-	# Magic regexp which hopefully exactly quotes backslashes and quotes
-	# Remi... $_[2] =~ /\G\s*"((?:[^"\\]|\\.)*)"\s*/gsc 
-
-	# $_[2] =~ /\G\s*\"((?:[^\"\\]|\\.)*)\"\s*/gsc
-	# This regexp is closer to the VRML97 specification:
-	$_[2] =~ /\G\s*\"((?:[^\"\\]|\\[\"\\])*)\"\s*/gsc
-		or VRML::Error::parsefail($_[2],"improper SFString");
-
+	$_[2] =~ /\G\s*\x22($Chars)\x22\s*/g
+		or VRML::Error::parsefail($_[2], "improper SFString");
 	my $str = $1;
+	## cleanup ms-dos/windows end of line chars
+	$str =~ s/
+//g;
 	$str =~ s/\\(.)/$1/g;
-
 	return $str;
 }
 
@@ -766,22 +571,6 @@ sub cfunc {"sv_setsv($_[1],$_[2]);"}
 sub print {print "\"$_[1]\""}
 sub as_string{"\"$_[1]\""}
 
-# JAVA SAI stuff...
-
-sub jalloc {""}
-sub jcopy {"v = f.getValue();"}
-sub jdata {"String v;"}
-sub jclonearg {"v"}
-#sub jset {return {""=>"v = \"\";", "String val" => "v=val;"}}
-sub jset {}
-sub jset_str { '
-	v = new String(v);
-'}
-sub jget {return {String => "return v;"}}
-sub jstr {"return String(v);"}
-
-# end of JAVA SAI stuff...
-
 
 ###########################################################
 package VRML::Field::MFString;
@@ -899,11 +688,37 @@ package VRML::Field::MFRotation;
 package VRML::Field::Multi;
 @ISA=VRML::Field;
 
+sub init { return []; }
+
 sub ctype {
 	my $r = (ref $_[0] or $_[0]);
 	$r =~ s/VRML::Field::MF//;
 	return "struct Multi_$r $_[1]";
 }
+sub clength {
+	#for C routes. Keep in sync with getClen in VRMLC.pm.
+	#clengths that are negative signal that something more than just a straight
+	#copy for javascripting is required. Other than the fact that numbers are
+	#negative, there is no symbolism placed on their values.
+
+	my $r = (ref $_[0] or $_[0]);
+	$r =~ s/VRML::Field::MF//;
+	# these negative ones are returned in other places...
+	# -11 = SFNode
+	# -12 = SFImage
+
+	if ($r eq "String") {return -13;}	# signal that a clength -13 is a Multi_String for CRoutes
+	if ($r eq "Float") {return -14;}	# signal that a clength -14 is a Multi_Float for CRoutes
+	if ($r eq "Rotation") {return -15;}	# signal that a clength -15 is a Multi_Rotation for CRoutes
+	if ($r eq "Int32") {return -16;}	# signal that a clength -16 is a Multi_Int32 for CRoutes
+	if ($r eq "Color") {return -17;}	# signal that a clength -1  is a Multi_Color for CRoutes
+	if ($r eq "Vec2f") {return -18;}	# signal that a clength -18 is a Multi_Vec2f for CRoutes
+	if ($r eq "Vec3f") {return -1;} 	# signal that a clength -1  is a Multi_Vec3f for CRoutes
+	if ($r eq "Node") {return -10;} 	# signal that a clength -10 is a Multi_Node for CRoutes
+
+	print "clength struct not handled Multi_$r $_[1]\n";
+	return 0;
+}
 sub cstruct {
 	my $r = (ref $_[0] or $_[0]);
 	my $t = $r;
@@ -1021,27 +836,26 @@ sub print {
 }
 
 sub as_string {
-	my $r = $_[0];
-	$r =~ s/::MF/::SF/;
-	if("ARRAY" eq ref $_[1]) {
-		"(Multi print, $_[0], $_[1])", @{$_[1]},"  [ ".(join ' ',map {$r->as_string($_)} @{$_[1]})." ] ";
+	my ($type, $value, $as_ecmascript) = @_;
+	$type =~ s/::MF/::SF/;
+
+	if (!$value) { return; }
+
+	if("ARRAY" eq ref $value) {
+		return "[]" if (!@{$value});
+		return "[ ".(join ' ', map {$type->as_string($_)} @{$value})." ]";
 	} else {
-		$_[1]->as_string();
+		return $value->as_string();
 	}
 }
 
 
-sub js_default {
-	my($type) = @_;
-	# $type =~ s/::MF/::SF/;
-	$type =~ s/VRML::Field:://;
-	return "new $type()";
-}
-
 
 ###########################################################
 package VRML::Field::SFNode;
 
+sub init { return "NULL"; }
+
 sub copy { return $_[1] }
 
 sub ctype {"void *$_[1]"}      # XXX ???
@@ -1057,13 +871,8 @@ sub cget {if(!defined $_[2]) {return "$_[1]"}
 sub as_string {
 	$_[1]->as_string();
 }
+sub clength {-11} #for C routes. Keep in sync with getClen in VRMLC.pm.
 
-sub js_default { 'new SFNode("","NULL")' }
-
-sub toj {$_[1]}
-sub fromj {$_[1]}
-
-# javascript implemented in place because of special nature.
 
 ##
 ## RCS: Implement SFImage
@@ -1076,191 +885,35 @@ package VRML::Field::SFImage;
 @ISA=VRML::Field;
 VRML::Error->import;
 
+sub init { return [0, 0, 0]; }
+
 sub parse {
   my($type,$p) = @_;
 
-  # JAS $VRML::verbose::parse=1;
+# define SFImage as being a range of "numbers" separated by spaces, tabs, newlines, etc.
+# this will end at what hopefully is the trailing brace.
 
-  $_[2] =~ /\G\s*([0-9]+)\s+([0-9]+)\s+([0-9]+)/ogcs
-    or parsefail($_[2], "didn't match width/height/depth of SFImage");
-  my ($sfimage_width, $sfimage_height, $sfimage_depth) = ($1, $2, $3);
-  parsefail($_[2], "pixel texture depth is invalid: consider values between 1 and 4 inclusive !")
-    if (! (1 <= $sfimage_depth && $sfimage_depth <= 4));
-  print "VRML::Field::SFImage::parse -  width = $sfimage_width     height = $sfimage_height   depth = $sfimage_depth \n"
-    if $VRML::verbose::parse;
-  my $npixgrp = ($sfimage_width * $sfimage_height) -1;
-  my @pixrows; # A table of sfimage_height rows of (sfimage_width * sfimage_depth) columns
-  my $x = 0, $y = 0;
- PIXGRP:
-  while (1) 
-    {
-      $_[2] =~ /\G\s+((0x)[0-9A-Fa-f]+|[0-9]+)/ogcs
-  	or parsefail($_[2], "didn't match a pixel (still $npixgrp to go)");
-      my $pixel = $1;
-      my $isx = (defined $2);
-      print "VRML::Field::SFImage::parse -  pixel is $1 (".($2 ? "hexa" : "decimal").")\n" 
-	if $VRML::verbose::parse;
-      if ($isx) 
-	{
- 	DEPTHSWITCHX:
-	  for ($sfimage_depth) {
-	    m/1/ && do { 
-  	      parsefail($pixel, "pixel $pixel didn't match a one-component pixel", "do not have correct number of digits") if ($pixel !~ m/0x([0-9A-Fa-f]{2})/);
-  	      $pixrows[$y][$x++] = sprintf("%02s", $1); 
-	      last DEPTHSWITCHX; 
-	    };
-	    m/2/ && do { 
-   	      parsefail($pixel, "pixel $pixel didn't match a two-component pixel", "do not have correct number of digits") if ($pixel !~ m/0x([0-9A-Fa-f]{2})([0-9A-Fa-f]{2})/);
-  	      $pixrows[$y][$x++] = sprintf("%02s", $1);
-  	      $pixrows[$y][$x++] = sprintf("%02s", $2);
-	      last DEPTHSWITCHX; 
-	    };
-	    m/3/ && do {
-   	      parsefail($pixel, "pixel $pixel didn't match a three-component pixel", "do not have correct number of digits") if ($pixel !~ m/0x([0-9A-Fa-f]{2}){3}/);
-  	      $pixel =~ m/0x([0-9A-Fa-f]{2})([0-9A-Fa-f]{2})([0-9A-Fa-f]{2})/;
-  	      $pixrows[$y][$x++] = sprintf("%02s", $1);
-  	      $pixrows[$y][$x++] = sprintf("%02s", $2);
-  	      $pixrows[$y][$x++] = sprintf("%02s", $3);
-	      last DEPTHSWITCHX; 
-	    };
-	    m/4/ && do {
-   	      parsefail($pixel, "pixel $pixel didn't match a three-component pixel", "do not have correct number of digits") if ($pixel !~ m/0x([0-9A-Fa-f]{2}){4}/);
-  	      $pixel =~ m/0x([0-9A-Fa-f]{2})([0-9A-Fa-f]{2})([0-9A-Fa-f]{2})([0-9A-Fa-f]{2})/;
-  	      $pixrows[$y][$x++] = sprintf("%02s", $1);
-  	      $pixrows[$y][$x++] = sprintf("%02s", $2);
-  	      $pixrows[$y][$x++] = sprintf("%02s", $3);
-  	      $pixrows[$y][$x++] = sprintf("%02s", $4);
-	      last DEPTHSWITCHX;
-	    };
-	  }
-	} 
-      else
-	{
- 	DEPTHSWITCH:
-	  for ($sfimage_depth) {
-	    m/1/ && do { 
-	      parsefail($pixel, "pixel $pixel didn't match a one-component pixel", "greater than 0xFF") if ($pixel >= 0x100);
-  	      $pixrows[$y][$x++] = sprintf("%x", $pixel); 
-	      last DEPTHSWITCH; 
-	    };
-	    m/2/ && do { 
- 	      parsefail($pixel, "pixel $pixel didn't match a two-component pixel", "greater than 0xFFFF") if ($pixel >= 0x10000);
-  	      $pixrows[$y][$x++] = sprintf("%02x", ((int($pixel) >> 8) & 0x00ff));	      
-  	      $pixrows[$y][$x++] = sprintf("%02x", (int($pixel) & 0x00ff));
-	      last DEPTHSWITCH; 
-	    };
-	    m/3/ && do {
- 	      parsefail($pixel, "pixel $pixel didn't match a three-component pixel", "greater than 0xFFFFFF") if ($pixel >= 0x1000000);
-  	      sprintf( "%06x", $pixel ) =~ m/([0-9A-Fa-f]{2})([0-9A-Fa-f]{2})([0-9A-Fa-f]{2})/;
-  	      $pixrows[$y][$x++] = $1;
-  	      $pixrows[$y][$x++] = $2;
-  	      $pixrows[$y][$x++] = $3;
-	      last DEPTHSWITCH; 
-	    };
-	    m/4/ && do {
- 	      parsefail($pixel, "pixel $pixel didn't match a three-component pixel", "greater than 0xFFFFFFFF") if ($pixel > 0xFFFFFFFF);
-  	      sprintf( "%06x", (($pixel >> 8) & 0x00ffffff)) =~ m/([0-9A-Fa-f]{2})([0-9A-Fa-f]{2})([0-9A-Fa-f]{2})/;
-  	      $pixrows[$y][$x++] = $1;
-  	      $pixrows[$y][$x++] = $2;
-  	      $pixrows[$y][$x++] = $3;
-  	      $pixrows[$y][$x++] = sprintf( "%02x", ($pixel & 0x000000ff));
-	      last DEPTHSWITCH;
-	    };
-	  }
-	}
-      do {$x = 0; $y++} if ($x == ($sfimage_width * $sfimage_depth));
-      next PIXGRP if ($npixgrp--);
-      last PIXGRP;
-    }
-  my $dat = '';
-  my $len = 0;
-  for $y (0 .. ($sfimage_height -1))
-    {
-      for $x (0 .. (($sfimage_width * $sfimage_depth) -1))
-	{
-  	  $dat = $dat.eval( 'chr(0x'.$pixrows[$y][$x].' )');
-	}
-    }
-  return [$sfimage_width, $sfimage_height, $sfimage_depth, $dat];
+$SFImageChars = qr~[a-fA-FxX+\- \n\t\s\d]+~;
+
+# now, get the image data
+$_[2] =~ /\G\s*($SFImageChars)\s*/gc
+	or parsefail ($_[2], " problem finding SFImage");
+  return $1;
 }
 
-sub print {print join ' ',@{$_[1]}}
-sub as_string {join ' ',@{$_[1]}}
 
-sub ctype {"struct SFImage $_[1]"}
-sub cstruct {return <<EOF
-struct SFImage {
-  int __x;
-  int __y;
-  int __depth;
-  unsigned char *__data;
-};
-EOF
-}
+sub ctype {return "SV *$_[1]"}
+sub calloc {"$_[1] = newSVpv(\"\",0);"}
+sub cassign {"sv_setsv($_[1],$_[2]);"}
+sub cfree {"SvREFCNT_dec($_[1]);"}
+sub cfunc {"sv_setsv($_[1],$_[2]);"}
 
-sub cget {return "($_[1].$_[2])"}
+sub print {print "\"$_[1]\""}
+sub as_string{"\"$_[1]\""}
 
-sub cfunc {
-	return <<EOF
-            {
-		AV *a;
-		SV **__data, **__x, **__y, **__depth;
-		STRLEN pl_na;
-		int i, x, y;
-		if(!SvROK($_[2])) {
-			$_[1].__data = NULL;
-			$_[1].__x = 0;
-			$_[1].__y = 0;
-			$_[1].__depth = 0;
-			/* die(\"Help! SFImage without being ref\"); */
-		} else {
-			if(SvTYPE(SvRV($_[2])) != SVt_PVAV) {
-				die(\"Help! SFImage without being arrayref\");
-			}
-			a = (AV *) SvRV($_[2]);
+sub clength {-12}; # signal that a -12 is a SFImage for CRoutes #for C routes. Keep in sync with getClen in VRMLC.pm.
 
-			/* __x */
-			__x = av_fetch(a, 0, 1); /* LVal for easiness */
-			if(!__x) die(\"Help: SFImage __x == NULL\");
-			$_[1].__x = SvNV(*__x);
-			/* fprintf(stdout, \"SFImage __x = %d\\n\", $_[1].__x); */
 
-			/* __y */
-			__y = av_fetch(a, 1, 1); /* LVal for easiness */
-			if(!__y) die(\"Help: SFImage __y == NULL\");
-			$_[1].__y = SvNV(*__y);
-			/* fprintf(stdout, \"SFImage __y = %d\\n\", $_[1].__y); */
-
-			/* __depth */
-			__depth = av_fetch(a, 2, 1); /* LVal for easiness */
-			if(!__depth) die(\"Help: SFImage __depth == NULL\");
-			$_[1].__depth = SvNV(*__depth);
-			/* fprintf(stdout, \"SFImage __depth = %d\\n\", $_[1].__depth); */
-
-			/* Handle image data */
-			__data = av_fetch(a, 4, 1); /* LVal for easiness */
-			if(!__data) die(\"Help: SFImage __data == 0\");
-			$_[1].__data = SvPV(*__data, pl_na);
-			/* fprintf(stdout, \"SFImage __data = %s  (len = %d) \\n\", $_[1].__data, pl_na); */
-		}
-	}
-EOF
-}
-
-sub js_ctor {
-  return <<EOF;
-    function SFImage (x, y, d, data) {
-      this.x = x
-      this.y = y
-      this.d = d
-      this.data = data
-    }
-EOF
-}
-
-sub js_default {
-  return "new SFImage(0,0,0,\"\")";
-}
 
 1;
 
