@@ -4,7 +4,6 @@
 
 package vrml.external;
 
-
 import java.util.*;
 import java.applet.*;
 import java.awt.*;
@@ -21,32 +20,54 @@ import vrml.external.FreeWRLEAI.EAIAsyncThread;
 import vrml.external.exception.InvalidNodeException;
 import vrml.external.exception.InvalidVrmlException;
 import vrml.external.BrowserGlobals;
-import netscape.security.*;
+import java.lang.System;
+//JAS - no longer using Netscape import netscape.security.*;
 
 public class Browser implements BrowserInterface
 
 {
-    // The thread that reads and processes FreeWRL EAI replies...
-     Thread 		FreeWRLThread;
 
-    // The Thread that sends stuff to the EAI port for FreeWRL...
-    static EAIoutThread 		EAIoutSender;
+	//====================================================================
+	// Threads:
+
+		// Replies from FreeWRL.
+		static Thread 		FreeWRLThread; // of type EAIinThread
+
+		// Send commands to FreeWRL.
+		static EAIoutThread 		EAIoutSender;
+
+		// Handle Async communications from FreeWRL (eg, Regisered Listeners)
+    		static EAIAsyncThread        RL_Async;
 
 
-    // The following are used to send to/from the FreeWLR Browser...
-    ServerSocket	EAISocket;
+	//====================================================================
+	// Communication Paths:
+
+		// FreeWRLThread to Browser - responses from commands to FreeWRL.
+    		PrintWriter EAIinThreadtoBrowser;
+    		PipedWriter EAIinThreadtoBrowserPipe = null;
+
+		// 
+    		static BufferedReader BrowserfromEAI = null;
+    		PipedReader BrowserfromEAIPipe = null;
+
+
+
+
+    // The following are used to send to/from the FreeWRL Browser by:
+    // 		- EAIinThread
+
+    Socket	EAISocket;
     Socket		sock;
     static PrintWriter         EAIout;
+    
 
     // The following pipe listens for replies to events sent to
     // the FreeWRL VRML viewer via the EAI port.
 
-    private PipedReader EAIfromFreeWRLStream;
-    static BufferedReader EAIfromFreeWRLInputStream;
 
     private String              reply = "";
 
-    static EAIAsyncThread        RL_Async;
     
     // Query Number as sent to the FreeWRL Browser.
     static int   queryno = 1;
@@ -72,19 +93,6 @@ public class Browser implements BrowserInterface
         return BrowserGlobals.EVObserver[eventno];
       }
 
-    public boolean get_Browser_EV_short_reply (int event)
-      {
-	int EVcounter;
-        for (EVcounter=0; EVcounter<BrowserGlobals.EVno; EVcounter++) {
-          if (BrowserGlobals.EVarray[EVcounter] == event) {
-            break;
-          }
-        }
-	// System.out.println ("get_Browser_EV_short_reply is returning " + BrowserGlobals.EVshortreply[EVcounter]);
-        return BrowserGlobals.EVshortreply[EVcounter];
-
-      }
-
     public void Browser_RL_Async_send (String EVentreply, int eventno) 
       {
         int EVcounter;
@@ -93,78 +101,89 @@ public class Browser implements BrowserInterface
             break;
           }
         }
-	// System.out.println ("Browser_RL_Async_send sending " + EVentreply + " to number " + EVcounter);
+	//System.out.println ("Browser_RL_Async_send sending " + EVentreply + " to number " + EVcounter);
         RL_Async.send(EVentreply, EVcounter);
       }
 
     // Associates this instance with the first embedded plugin in the current frame.
     public Browser(Applet pApplet) {
+	int counter;
 
   	// Create a socket here for an EAI server on localhost
 	int incrport = -1;
 	EAISocket = null;
 
-	// enable privleges
-	try {
-		PrivilegeManager.enablePrivilege ("UniversalConnect");
-	} catch (Throwable e) {
-		System.out.println("EAI: not using Netscape");
+	//System.out.println ("trying to open socket on 9877");
+	//enable privleges
+	//JAS try {
+	//JAS 	PrivilegeManager.enablePrivilege ("UniversalConnect");
+	//JAS } catch (Throwable e) {
+	//JAS 	System.out.println("EAI: not using Netscape");
+	//JAS }
+
+	counter = 1;	
+	while (EAISocket == null) {
+		try {
+			EAISocket = new Socket("localhost",9877);
+		} catch (IOException e) {
+			// wait up to 30 seconds for FreeWRL to answer.
+			counter = counter + 1;
+			if (counter == 60) {
+				System.out.println ("EAI: Java code timed out finding FreeWRL");
+				System.exit(1);
+			}
+			try {Thread.sleep (500);} catch (InterruptedException f) { }
+		}
 	}
-	
-	try {
-		EAISocket = new ServerSocket(2000);
-	} catch (IOException e) {
-		System.out.println ("EAI: Error creating socket for FreeWRL EAI on port 2000");
-	}
-	System.out.println ("EAI: opened port, ready for data" );
+
+	sock = EAISocket;	//JAS - these are the same now...
 
 
-  	try {
-  		sock=EAISocket.accept();
-  	} catch (IOException e) {
-  	  System.out.print ("EAI: System error on accept method\n");
-  	}
-  	// Start the readfrom FREEWRL thread...
-   	FreeWRLThread = new Thread ( new EAIinThread(sock, pApplet, this));
-           FreeWRLThread.start();
-  
+
+	//===================================================================
+	// create the EAIinThread to Browser.
+
   	// Open the pipe for EAI replies to be sent to us...
         try {
-          EAIfromFreeWRLStream = new PipedReader (EAIinThread.EAItoBrowserStream);
+		EAIinThreadtoBrowserPipe = new PipedWriter();
+		BrowserfromEAIPipe = new PipedReader(EAIinThreadtoBrowserPipe);
         } catch (IOException ie) {
           System.out.println ("EAI: caught error in new PipedReader: " + ie);
         }
-		EAIfromFreeWRLInputStream = new BufferedReader (EAIfromFreeWRLStream);	
-		
-		//System.out.println("waiting for thread...");
 
-  	// Wait for the FreeWRL browser to send us something...
-        try {
-          String fwvers = EAIfromFreeWRLInputStream.readLine());
-	  System.out.println ("EAI: FreeWRL Version: " + fwvers);
-        } catch (IOException ie) {System.out.println ("EAI: caught " + ie);}
-  
-	//System.out.println("got response from thread");
-  	// Send the correct response...
-	try {
-		EAIout = new PrintWriter (sock.getOutputStream());
-		EAIout.print ("FreeWRL EAI Serv0.27");
-		EAIout.flush ();
-	} catch (IOException e) {
-		System.out.print ("EAI: Problem in handshaking with Browser");
-	}
-	// System.out.println("browser is gotten");
-  	// Browser is "gotten", and is started.
+	EAIinThreadtoBrowser = new PrintWriter(EAIinThreadtoBrowserPipe);
+	BrowserfromEAI = new BufferedReader (BrowserfromEAIPipe);	
 
-  	// Start the SendTo FREEWRL thread...
-	EAIoutSender = new EAIoutThread(EAIout);
-        EAIoutSender.start();
+  	// Start the readfrom FREEWRL thread...
+   	FreeWRLThread = new Thread ( new EAIinThread(sock, pApplet, 
+		EAIinThreadtoBrowser, this));
+        FreeWRLThread.start();
 
+
+	//====================================================================
 	// Start the thread that allows Registered Listenered
 	// updates to come in.
 	RL_Async = new EAIAsyncThread();
 	RL_Async.start();
 
+  
+	//====================================================================
+	// create the EAIoutThread - send data to FreeWRL.
+	try {
+		EAIout = new PrintWriter (sock.getOutputStream());
+	} catch (IOException e) {
+		System.out.print ("EAI: Problem in handshaking with Browser");
+	}
+
+
+  	// Start the SendTo FREEWRL thread...
+	EAIoutSender = new EAIoutThread(EAIout);
+        EAIoutSender.start();
+
+
+
+	//====================================================================
+  	// Browser is "gotten", and is started.
   	return;
     }
   
@@ -175,46 +194,59 @@ public class Browser implements BrowserInterface
   
     }
   
+  // Get the browser name
+  public static String getName() {
+      String retval;
+       synchronized (FreeWRLToken) {
+         EAIoutSender.send ("" + queryno + "K\n");
+         retval = getVRMLreply(queryno);
+         queryno += 1;
+       }
+      return retval;
+  }
+
     public String        getVersion() {
-       return "0.28";
+       String retval;
+       synchronized (FreeWRLToken) {
+         EAIoutSender.send ("" + queryno + "L\n");
+         retval = getVRMLreply(queryno);
+         queryno += 1;
+       }
+      return retval;
      }
   
     // Get the current velocity of the bound viewpoint in meters/sec,
-    // if available, or 0.0 if not
     public float         getCurrentSpeed() {
-      System.out.println ("EAI: getCurrentSpeed Not Implemented");
-  
-      return (float) 0.0;
+       String retval;
+       synchronized (FreeWRLToken) {
+         EAIoutSender.send ("" + queryno + "M\n");
+         retval = getVRMLreply(queryno);
+         queryno += 1;
+       }
+      return Float.valueOf(retval).floatValue();
     }
   
     // Get the current frame rate of the browser, or 0.0 if not available
     public float         getCurrentFrameRate() {
-  
     String retval;
-
     synchronized (FreeWRLToken) {
-      EAIoutSender.send ("" + queryno + "\nGCFR\n");
+      EAIoutSender.send ("" + queryno + "N\n");
       retval = getVRMLreply(queryno);
       queryno += 1;
     }
-
     return Float.valueOf(retval).floatValue();
     }
   
     // Get the URL for the root of the current world, or an empty string
     // if not available
     public String        getWorldURL() {
-
       String retval;
-
-      
        synchronized (FreeWRLToken) {
-         EAIoutSender.send ("" + queryno + "\nGWU\n");
+         EAIoutSender.send ("" + queryno + "O\n");
          retval = getVRMLreply(queryno);
          queryno += 1;
        }
       return retval;
-
     }
   
     // Replace the current world with the passed array of nodes
@@ -230,7 +262,7 @@ public class Browser implements BrowserInterface
 	}
 
         synchronized (FreeWRLToken) {
-          EAIoutSender.send ("" + queryno + "\nRW" + SysString);
+          EAIoutSender.send ("" + queryno + "RW" + SysString);
           retval = getVRMLreply(queryno);
           queryno += 1;
         }
@@ -261,18 +293,22 @@ public class Browser implements BrowserInterface
     public Node[]        createVrmlFromString(String vrmlSyntax) 
 			throws InvalidVrmlException {
 
-      Node[]  x = new Node[2000];
+      Node[]  x = new Node[100];
       StringTokenizer tokens;
       String retval;
       String temp;
       int count;
 
       synchronized (FreeWRLToken) {
-        EAIoutSender.send ("" +queryno + "\nCVS "+vrmlSyntax+"\nEOT\n");
+        EAIoutSender.send ("" +queryno + "S "+vrmlSyntax+"\nEOT\n");
         retval = getVRMLreply(queryno);
 
+	// System.out.println ("Browser.java - got " + retval);
+
         tokens = new StringTokenizer (retval);
-	x = new Node[tokens.countTokens()];
+
+	// We return a (node-index C-pointer) pair, so we have to divide by two
+	x = new Node[tokens.countTokens()/2];
         count = 0;
 
 	// Lets go through the output, and temporarily store it
@@ -280,15 +316,23 @@ public class Browser implements BrowserInterface
         while (tokens.hasMoreTokens()) {
           x[count] = new Node();
           x[count].NodeName = tokens.nextToken();
+          x[count].nodeptr = tokens.nextToken();
+          x[count].offset = "0";
+          x[count].datasize = "4";
+          x[count].datatype = "h"; // see CFuncs/EAIServ.c for a complete desc.
+
+	  //System.out.println ("CVS - for this one, we have NODE" + x[count].NodeName +
+	//	" pointer:" + x[count].nodeptr + " offset:" + x[count].offset +
+	//	" datasize: " + x[count].datasize + " datatype:" + x[count].datatype);
+
           count ++;
-	  if (count == 2000) {
-		count = 1999;
-		System.out.println ("EAI: createVrmlFromString - warning, tied to 2000 nodes");
+	  if (count == 100) {
+		count = 99;
+		System.out.println ("EAI: createVrmlFromString - warning, tied to 100 nodes");
 	  }
         }
         queryno += 1;
       }
-
       return x;
     }
   
@@ -309,17 +353,25 @@ public class Browser implements BrowserInterface
 
 
        synchronized (FreeWRLToken) {
-         EAIoutSender.send ("" + queryno + "\nCVU " + url[0] + "\n");
+         EAIoutSender.send ("" + queryno + "T " + url[0] + "\n");
 
          retval = getVRMLreply(queryno);
 
          tokens = new StringTokenizer (retval);
          count = 0;
- 	 x = new Node[tokens.countTokens()];
+ 	 x = new Node[tokens.countTokens()/2];
 
          while (tokens.hasMoreTokens()) {
            x[count] = new Node();
            x[count].NodeName = tokens.nextToken();
+           x[count].nodeptr = tokens.nextToken();
+           x[count].offset = "0";
+           x[count].datasize = "4";
+           x[count].datatype = "h"; // see CFuncs/EAIServ.c for a complete desc.
+	   //System.out.println ("CVU - for this one, we have NODE" + x[count].NodeName +
+		//	" pointer:" + x[count].nodeptr + " offset:" + x[count].offset +
+		//	" datasize: " + x[count].datasize + " datatype:" + x[count].datatype);
+
            count ++;
          }
  	queryno += 1;
@@ -336,14 +388,30 @@ public class Browser implements BrowserInterface
     public void          addRoute(Node fromNode, String fromEventOut,
                                   Node toNode, String toEventIn) throws
 				  IllegalArgumentException {
-      throw new IllegalArgumentException ("AddRoute Not Implemented");
+      String retval;
+
+      synchronized (FreeWRLToken) {
+         EAIoutSender.send ("" + queryno + "H " + fromNode + " " + fromEventOut +
+		" " + toNode + " " + toEventIn +"\n");
+         retval = getVRMLreply(queryno);
+         queryno += 1;
+       }
+      return;
     }
   
   
     public void          deleteRoute(Node fromNode, String fromEventOut,
                                      Node toNode, String toEventIn) 
 			 throws IllegalArgumentException {
-      throw new IllegalArgumentException ("DeleteRoute Not Implemented");
+      String retval;
+
+      synchronized (FreeWRLToken) {
+         EAIoutSender.send ("" + queryno + "J " + fromNode + " " + fromEventOut +
+		" " + toNode + " " + toEventIn +"\n");
+         retval = getVRMLreply(queryno);
+         queryno += 1;
+       }
+      return;
     }
   
     // begin and end an update cycle
@@ -393,7 +461,7 @@ public class Browser implements BrowserInterface
       temp = new Node();  
 
       synchronized (FreeWRLToken) {
-        EAIoutSender.send ("" + queryno + "\nGN " + NodeName + "\n");
+        EAIoutSender.send ("" + queryno + "A " + NodeName + "\n");
         temp.NodeName = getVRMLreply (queryno);
         queryno += 1;
       }
@@ -406,123 +474,87 @@ public class Browser implements BrowserInterface
 
 
 
-  //
-  // Send Event to the VRML Browser. Note the different methods, depending
-  // on the parameters.
-  //
-  // SendChildEvent waits for confirmation that child is added/removed to MFNode array.
-  // This gets around the problem of sending two adds in succession, and having
-  // the second overwrite the first.
-  public static void SendChildEvent (String NodeName, String FieldName, String Value)
-    {
-      String retval;
+	//
+	// Send Event to the VRML Browser. Note the different methods, depending
+	// on the parameters.
+	//
+	public static void SendChildEvent (String parent, String offset, String FieldName, String Child) {
+		String retval;
 
-      synchronized (FreeWRLToken) {
-        EAIoutSender.send ("" + queryno + "\nSC " + NodeName + " " + 
-           FieldName + "\n" + Value + "\n");
-        retval = getVRMLreply(queryno);
-        queryno += 1;
+		// System.out.println ("SendChildEvent: sending to " + parent + " ofs:" + 
+		//	offset + " field " + FieldName + " child: " + Child);
+
+		synchronized (FreeWRLToken) {
+			EAIoutSender.send ("" + queryno + "C " + parent + " " + offset + " " +
+				FieldName + " "+ Child + "\n");
+			retval = getVRMLreply(queryno);
+			queryno += 1;
+		}
+		return;
+	}
+
+	// Most events don't need us to wait around for it.
+	public static void newSendEvent (Node node, String Value) {
 	
-        // Now, tell FreeWRL to update routes
-        EAIoutSender.send ("" + queryno + "\nUR " + NodeName + " " + 
-           FieldName + "\n" + Value + "\n");
-        retval = getVRMLreply(queryno);
-        queryno += 1;
-      }
-      return;
-    }
-
-  // Most events don't need us to wait around for it.
-  public static void SendEvent (String NodeName, String FieldName, String Value)
-    {
-
-      synchronized (FreeWRLToken) {
-        EAIoutSender.send ("" + queryno + "\nSE " + NodeName + " " + 
-           FieldName + "\n" + Value + "\n");
-        // JAS - don't wait. getVRMLreply(queryno);
-        queryno += 1;
-      }
-      return;
-    }
-
-  // Get the browser name
-  public static String getName() {
-      String retval;
-
-       synchronized (FreeWRLToken) {
-         EAIoutSender.send ("" + queryno + "\nGNAM\n");
-         retval = getVRMLreply(queryno);
-         queryno += 1;
-       }
-      return retval;
-  }
+		//System.out.println ("SendEvent, sending NodeType " + NodeType + " NodePtr " + NodePtr +
+		//	" Offset " + Offset);
+		synchronized (FreeWRLToken) {
+			EAIoutSender.send ("" + queryno + "D" + node.datatype + " " + 
+				node.nodeptr + " " + node.offset + " " +node.ScriptType + " " + Value + "\n");
+			queryno += 1;
+		}
+		return;
+	}
 
 
-  protected static String SendEventType (String NodeName, String FieldName) {
+  protected static String SendEventType (String NodeName, String FieldName, String direction) {
+
       // get a type from a particular node.
 
       String retval;
 
       synchronized (FreeWRLToken) {
-        EAIoutSender.send ("" + queryno + "\nGT " + NodeName + " " +
-           FieldName + "\n");
+        EAIoutSender.send ("" + queryno + "F " + NodeName + " " +
+           FieldName + " " + direction + "\n");
         retval = getVRMLreply(queryno);
         queryno += 1;
       }
       return retval;
 }
 
-  public static String SendEventOut (String NodeName, String FieldName) {
+  public static String SendEventOut (String nodeptr, String offset, String datasize, String datatype,
+	String command) {
       // get a value from a particular node.
 
-      String retval;
+	//System.out.println ("SendEventOut, nodeptr " + nodeptr + " datatype " + datatype);
 
+      String retval;
        synchronized (FreeWRLToken) {
-         EAIoutSender.send ("" + queryno + "\nGV " + NodeName + " " +
-            FieldName + "\n");
+         EAIoutSender.send ("" + queryno + "E " + nodeptr + " " + offset + " " + datatype +
+ 		" " + datasize + "\n");
+ 
          retval = getVRMLreply(queryno);
          queryno += 1;
       }
+	//System.out.println ("SendEventOut returns " + retval);
      return retval;
 }
 
   public static void RegisterListener (EventOutObserver f, Object userData,
-			String outNode, String command,  int EventType)
+			String nodeptr, String offset, String datatype,  String datasize, int EventType)
     {
        synchronized (FreeWRLToken) {
-         EAIoutSender.send ("" + queryno + "\nRL " + outNode + 
- 		" " + command + " " + queryno + "\n");
+         	//System.out.println ("Registering " + queryno + " node: " + nodeptr + " offset:"
+		// + offset + " datatype:" + datatype +
+ 		//" datasize:" + datasize + "\n");
+
+         EAIoutSender.send ("" + queryno + "G " + nodeptr + " " + offset + " " + datatype +
+ 		" " + datasize + "\n");
  
          BrowserGlobals.EVarray [BrowserGlobals.EVno] =  queryno;
          BrowserGlobals.EVtype [BrowserGlobals.EVno] = EventType;     
          BrowserGlobals.EVObject[BrowserGlobals.EVno] = userData;
          BrowserGlobals.EVObserver[BrowserGlobals.EVno] = f;
-	 // Is this a short, consise answer type? 
-	 // (see field/FieldTypes.java for more info)
-
-	 switch (EventType) {
-		case 1:
-		case 3:
-		case 4:
-		case 5:
-		case 6:
-		case 7:
-		case 8:
-		case 9:
-		case 12:
-		case 13:
-		case 14:
-		case 15:
-		case 16:
-		case 17:
-		case 18:
-		case 19:
-			BrowserGlobals.EVshortreply[BrowserGlobals.EVno] = true;
-			break;
-		default:
-			BrowserGlobals.EVshortreply[BrowserGlobals.EVno] = false;
-	 } 
-
 
          BrowserGlobals.EVno += 1;
        
@@ -544,7 +576,7 @@ public class Browser implements BrowserInterface
  
  	while (queryno != Integer.parseInt(req)) { 
            try {
-             req = Browser.EAIfromFreeWRLInputStream.readLine();
+             req = BrowserfromEAI.readLine();
            } catch (IOException ie) {
 		System.out.println ("EAI: caught " + ie);
 		return rep;
@@ -557,7 +589,7 @@ public class Browser implements BrowserInterface
            }
        
            try {
-                 rep = Browser.EAIfromFreeWRLInputStream.readLine(); 
+                 rep = BrowserfromEAI.readLine(); 
            } catch (IOException ie) { System.out.println ("EAI: getVRMLreply failed"); return null; }
 	
 
@@ -567,12 +599,14 @@ public class Browser implements BrowserInterface
       }  
 	public void close() {
 		System.out.println("EAI: closing socket");
-		try {
-			EAIoutSender.stopThread();
-			EAISocket.close();
-			EAIfromFreeWRLStream.close();
-		} catch (IOException e) {
-		}
+		//JAS try {
+			System.exit(1);
+			//JAS EAIoutSender.stopThread();
+			//JAS EAISocket.close();
+			//JAS EAIfromFreeWRLStream.close();
+  
+		//JAS } catch (IOException e) {
+		//JAS }
 	}
 }
 
