@@ -27,10 +27,17 @@ public class VSFInt32 extends VField
 	value = in.readInt();
     }
 
+    public VSFInt32(int v) 
+    {
+	value = v;
+    }
+
     public void write(DataOutputStream out) throws IOException
     {
 	out.writeInt(value);
     }
 
+    public int getValue() { return value;}
+
     public byte getType() { return SFINT32; }
 }
