@@ -11,14 +11,13 @@ public class EventInMFNode extends EventIn {
     int count;
 
     for (count = 0; count < node.length; count++) {
-      Browser.SendChildEvent (inNode , command, node[count].NodeName);
+      Browser.SendChildEvent (nodeptr,offset, command, node[count].nodeptr);
     }
   return;
   }
 
   public void          set1Value(int index, Node node) {
-    System.out.println ("ERROR: eventinmfnode.set1vanue Not Implemented");
-    // Browser.SendEvent (inNode , command, "TRUE");
+      Browser.SendChildEvent (nodeptr,offset, command, node.nodeptr);
 
   return;
   }
