@@ -7,9 +7,9 @@ public class EventInSFBool extends EventIn {
 
   public void          setValue(boolean value) {
   if (value) {
-    Browser.SendEvent (inNode , command, "TRUE");
+    Browser.newSendEvent (this, "TRUE");
   } else {
-    Browser.SendEvent (inNode, command, "FALSE");
+    Browser.newSendEvent (this, "FALSE");
   }
   return;
   }
